package com.didana.codecontract.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * An annotation that supports Design by Contract "invariant" tests.
 * An invariant test on a field or method, including constructors,
 * will be evaluated before and after all method invocations.
 * <br>The value String must be a valid JEXL expression
 * evaluating to true or false at runtime.
 * The class must be annotated with @Contract
 * 
 * @author Diana Berberova
 * 
 */
@Documented
@Inherited
@Retention(RetentionPolicy.RUNTIME)
@Target({ElementType.TYPE, ElementType.ANNOTATION_TYPE, ElementType.METHOD, 
		 ElementType.CONSTRUCTOR, ElementType.FIELD})
public @interface Invar {
    /**
     * The "value" is the test expression, which must be a legal JEXL
     * expression that evaluates to true or false.
     * The default value is "".
     */
    String value() default "";

}
