package com.didana.codecontract.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * An annotation that supports Design by Contract "postcondition" tests.
 * A method or constructor with a postcondition annotation will have the test 
 * performed after the method executes.
 * <br/>The value String must be a valid JEXL expression
 * evaluating to true or false at runtime.
 * The class must be annotated with @Contract
 * 
 * @author Diana Berberova
 * 
 */
@Documented
@Retention(RetentionPolicy.RUNTIME)
@Target({ElementType.METHOD, ElementType.CONSTRUCTOR})
public @interface Post {
    /**
     * The "value" is the test expression, which must be a legal JEXL
     * expression that evaluates to true or false.
     * The default value is "".
     */
    String value() default "";

}
