package com.didana.codecontract.core;

import org.apache.commons.jexl.*;

import com.didana.codecontract.util.LogUtillity;

/**
 * The class represents a single contract test defined as 
 * precondition, postcondition or invariant.
 * The expressionString String must be a valid JEXL expression
 * evaluating to true or false at runtime.
 * 
 * @author Diana Berberova
 *
 */
public class Expression {
	private org.apache.commons.jexl.Expression jexlExpression;
	private static LogUtillity logger  = new LogUtillity(Expression.class);
	
	public Expression(String expressionString) {
		try {

			logger.debug("Trying to create expression: " + expressionString);
			jexlExpression = ExpressionFactory.createContractExpression(expressionString);
			logger.debug("Successfully created expression: " + expressionString);
			
		} catch (Exception e) {
			logger.error("Unexpected exception during initialization of expression: ", e);
			e.printStackTrace();
		}
	}
	
	
	public boolean evaluate(JexlContext jexlContext) {
		boolean result = false;
		try {
			if (jexlExpression.getExpression() != "") {
				logger.debug("Trying to evaluate expression: " + jexlExpression.getExpression() );
				logger.debug("Expression variables : " + jexlContext.getVars());

				Object obj = jexlExpression.evaluate(jexlContext);
				
				if (obj != null) {
					result = (obj.toString().equals("true"));
				}
				
				logger.debug("Expression evaluates to: " + result);
				
			} else { 
				logger.debug("Trying to evaluate empty expression.");
				result = true;
			}
			
		} catch (Exception e) {
			logger.error("Unexpected exception during evaluation of expression: ", e);	
		}
		
		return result;		
	}
}
