package com.didana.codecontract.core;

import com.didana.codecontract.util.LogUtillity;
import java.util.ArrayList;
import java.util.Map;
import org.apache.commons.jexl.JexlContext;
import org.apache.commons.jexl.JexlHelper;

/**
 * This class represents the context in which all contracts 
 * are evaluated during a runtime check of precondition/postcondition. 
 * 
 * The expressionStrings ArrayList<String> must be a list of 
 * Strings representing the contracts that should be checked.
 * 
 * The variables Map contains the values of all variables 
 * in the context of which the expressions expressionStrings 
 * should be evaluated.
 * 
 * 
 * @author Diana Berberova
 *
 */
public class ExpressionHandler {
    private JexlContext jexlContext;
    private Expression expressions[];
    
    private static LogUtillity logger = new LogUtillity(ExpressionHandler.class);
 	
    /**
     * Creates new Expression handler object.
     * 
     * @param expressionStrings - the expression strings that should be evaluated.
     * @param variables - the variables that take part in evaluation   
     */
    public ExpressionHandler(ArrayList<String> expressionStrings, Map variables) {
        try {
            int count = expressionStrings.size();
            expressions = new Expression[count];
            for(int i = 0; i < count; i++) {
    			expressions[i] = new Expression((String)expressionStrings.get(i));
            }

            jexlContext = JexlHelper.createContext();
            jexlContext.setVars(variables);
            
        } catch(Exception e) {
            logger.error("Unexpected exception during initialization of expression context: ", e);
            e.printStackTrace();
        }
    }

    /**
     * Evaluates the expressions in the current jexl context.
     *  
     * @return true if all the expressions 
     * are evaluated true and false otherwise.
     */
    public boolean evaluateExpressions() {
    	boolean result = true;
        try {
            for(int i = 0; i < expressions.length; i++) {
                result &= expressions[i].evaluate(jexlContext);
            }
        } catch(Exception e) {
            logger.error("Unexpected exception during evaluation of expressions: ", e);
            result = false;
        }
        
        return result;
    }
}
