package com.didana.codecontract.tests;

import static org.junit.Assert.*;

import java.util.ArrayList;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.didana.codecontract.annotation.*;
import com.didana.codecontract.core.ContractException;
import com.didana.codecontract.core.ContractHandler;

/**
 * JUnit test case for testing ContractHandler methods.
 * 
 * @author Diana Berberova
 *
 */
public class ContractHandlerTests {

	private static Class animalClass = Animal.class;
	private static Class mammalClass = Mammal.class;
	private static Class humanClass = Human.class;
	
	private static Class preClass = Pre.class;
	private static Class postClass = Post.class;
	
	private static ArrayList<String> animalInvariants = new ArrayList<String>(); 
	private static ArrayList<String> mammalInvariants = new ArrayList<String>(); 
	private static ArrayList<String> humanInvariants = new ArrayList<String>(); 

	private static ArrayList<String> animalInvariantContracts = new ArrayList<String>(); 
	private static ArrayList<String> mammalInvariantContracts = new ArrayList<String>(); 
	private static ArrayList<String> humanInvariantContracts = new ArrayList<String>(); 

	private static ArrayList<String> animalSetNumberOfLegsContractsPre = new ArrayList<String>(); 

	private static ArrayList<String> mammalSetNumberOfFingersContractsPost = new ArrayList<String>(); 
	private static ArrayList<String> mammalSetNumberOfLegsContractsPre = new ArrayList<String>(); 

	
	private static ArrayList<String> humanSetNumberOfFingersContractsPost = new ArrayList<String>(); 
	private static ArrayList<String> humanSetNumberOfLegsContractsPre = new ArrayList<String>(); 
	private static ArrayList<String> humanSetHumanNameContractsPost = new ArrayList<String>(); 
	private static ArrayList<String> humanSetHumanNameContractsPre = new ArrayList<String>(); 
	private static ArrayList<String> humanGetFullNameContractsPost = new ArrayList<String>(); 

	private static ArrayList<String> humanProtectedSetPost = new ArrayList<String>(); 
	private static ArrayList<String> humanPrivateSetPre= new ArrayList<String>(); 
	private static ArrayList<String> humanPackageSetPost = new ArrayList<String>(); 

	private static ArrayList<String> humanConstructorPre= new ArrayList<String>(); 
	private static ArrayList<String> humanConstructorPost = new ArrayList<String>(); 

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		//expected invariants defined in Animal interface
		animalInvariants.add("speciesName.equals('Animal')");

		//expected invariants defined in Mammal class
		mammalInvariants.add("description != null && description != ''");
		
		//expected invariants defined in Human class
		humanInvariants.add("numberTeeth >= 0 && numberTeeth <= 32");
		
		//all expected invariants checked for Animal interface
		animalInvariantContracts.addAll(animalInvariants);

		//all expected invariants checked for Mammal class
		mammalInvariantContracts.addAll(animalInvariants);
		mammalInvariantContracts.addAll(mammalInvariants);
		
		//all expected invariants checked for Human class
		humanInvariantContracts.addAll(animalInvariants);
		humanInvariantContracts.addAll(mammalInvariants);
		humanInvariantContracts.addAll(humanInvariants);
		
		//expected precondition contracts for Animal.setNumberOfLegs()
		animalSetNumberOfLegsContractsPre.addAll(animalInvariantContracts);
		animalSetNumberOfLegsContractsPre.add("$args0 >= 0");
		
		//expected precondition contracts for Mammal.setNumberOfFingers()
		mammalSetNumberOfFingersContractsPost.addAll(mammalInvariantContracts);
		mammalSetNumberOfFingersContractsPost.add("fingers >= 0");
		
		//expected precondition contracts for Mammal.setNumberOfLegs()
		mammalSetNumberOfLegsContractsPre.addAll(mammalInvariantContracts);
		
		//expected postcondition contracts for Human.setNumberOfFingers()
		humanSetNumberOfFingersContractsPost.addAll(humanInvariantContracts);
		humanSetNumberOfFingersContractsPost.add("fingers >= 0");
		
		//expected precondition contracts for Human.setNumberOfLegs()
		humanSetNumberOfLegsContractsPre.addAll(humanInvariantContracts);
		
		//expected postcondition contracts for Human.setHumanName()
		humanSetHumanNameContractsPost.addAll(humanInvariantContracts);
		humanSetHumanNameContractsPost.add("subspeciesName.equals('Mammal')");
		
		//expected precondition contracts for Human.setHumanName()
		humanSetHumanNameContractsPre.addAll(humanInvariantContracts);
		humanSetHumanNameContractsPre.add("$args0 != null && $args1 != ''");
		
		//expected postcondition contracts for Human.getFullName()
		humanGetFullNameContractsPost.addAll(humanInvariantContracts);
		humanGetFullNameContractsPost.add("$return.endsWith('Human')");
		humanGetFullNameContractsPost.add("$return.indexOf('Mammal') > -1");
		humanGetFullNameContractsPost.add("$return.startsWith('Animal')");

		//expected postcondition contracts for Human.protectedSet()
		humanProtectedSetPost.add("isProtected == true");

		//expected precondition contracts for Human.privateSet()
		humanPrivateSetPre.add("isPrivate == true");
		
		//expected postcondition contracts for Human.packageSet()
		humanPackageSetPost.add("isPackage == true");
		
		//expected precondition contracts for Human()
		humanConstructorPre.add("isConstructorPre == true");
		
		//expected postcondition contracts for Human()
		humanConstructorPost.add("isConstructorPost == true");
		humanConstructorPost.addAll(humanInvariantContracts);
	}
	

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Test
	public void testGetInvariantsAnimal() {
		try {
			ArrayList<String> result = ContractHandler.getClassInvariants(animalClass);
			assertEquals(animalInvariants, result);
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());			
		}		
	}

	@Test
	public void testGetInvariantsMammal() {
		try {
			ArrayList<String> result = ContractHandler.getClassInvariants(mammalClass);
			assertEquals(mammalInvariants, result);
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());			
		}		
	}
	
	@Test
	public void testGetInvariantsHuman() {
		try {
			ArrayList<String> result = ContractHandler.getClassInvariants(humanClass);
			assertEquals(humanInvariants, result);
		} catch (ContractException ce) {
			fail("ContractException thrown.");
		}		
	}
	
	@Test
	public void testGetContractAnimal1() {
		try {
			String signature = "public void com.didana.codecontract.tests.Animal.setNumberOfLegs(int)";
			ArrayList<String> result = ContractHandler.getContracts(animalClass, preClass, signature, true);
			assertTrue(animalSetNumberOfLegsContractsPre.containsAll(result) && result.containsAll(animalSetNumberOfLegsContractsPre));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());			
		}		
	}
	
	@Test
	public void testGetContractAnimal2() {
		try {
			String signature = "public void com.didana.codecontract.tests.Animal.setNumberOfLegs(int)";
			ArrayList<String> result = ContractHandler.getContracts(animalClass, postClass, signature, true);
			assertTrue(animalInvariantContracts.containsAll(result) && result.containsAll(animalInvariantContracts));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());			
		}		
	}
	
	@Test
	public void testGetContractMammal1() {
		try {
			String signature = "public void com.didana.codecontract.tests.Mammal.setNumberOfFingers(int)";
			ArrayList<String> result = ContractHandler.getContracts(mammalClass, postClass, signature, true);
			assertTrue(mammalSetNumberOfFingersContractsPost.containsAll(result) && result.containsAll(mammalSetNumberOfFingersContractsPost));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());						
		}		
	}
	
	@Test
	public void testGetContractMammal2() {
		try {
			String signature = "public void com.didana.codecontract.tests.Mammal.setNumberOfFingers(int)";
			ArrayList<String> result = ContractHandler.getContracts(mammalClass, preClass, signature, true);
			assertTrue(mammalInvariantContracts.containsAll(result) && result.containsAll(mammalInvariantContracts));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());					
		}		
	}
	
	@Test
	public void testGetContractMammal3() {
		try {
			String signature = "public void com.didana.codecontract.tests.Mammal.setNumberOfLegs(int)";
			ArrayList<String> result = ContractHandler.getContracts(mammalClass, preClass, signature, true);
			assertTrue(mammalSetNumberOfLegsContractsPre.containsAll(result) && result.containsAll(mammalSetNumberOfLegsContractsPre));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage() + ce.getInnerMessage());			
		}		
	}


	@Test
	public void testGetContractMammal4() {
		try {
			String signature = "public void com.didana.codecontract.tests.Mammal.setNumberOfLegs(int)";
			ArrayList<String> result = ContractHandler.getContracts(mammalClass, postClass, signature, true);
			assertTrue(mammalInvariantContracts.containsAll(result) && result.containsAll(mammalInvariantContracts));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());						
		}		
	}
	
	@Test
	public void testGetContractHuman1() {
		try {
			String signature = "public void com.didana.codecontract.tests.Human.setNumberOfLegs(int)";
			ArrayList<String> result = ContractHandler.getContracts(humanClass, preClass, signature, true);
			assertTrue(humanSetNumberOfLegsContractsPre.containsAll(result) && result.containsAll(humanSetNumberOfLegsContractsPre));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());						
		}		
	}
	
	@Test
	public void testGetContractHuman2() {
		try {
			String signature = "public void com.didana.codecontract.tests.Human.setNumberOfFingers(int)";
			ArrayList<String> result = ContractHandler.getContracts(humanClass, postClass, signature, true);
			assertTrue(humanSetNumberOfFingersContractsPost.containsAll(result) && result.containsAll(humanSetNumberOfFingersContractsPost));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());						
		}		
	}
	
	@Test
	public void testGetContractHuman3() {
		try {
			String signature = "public void com.didana.codecontract.tests.Human.setHumanName(java.lang.String)";
			ArrayList<String> result = ContractHandler.getContracts(humanClass, postClass, signature, true);
			assertTrue(humanSetHumanNameContractsPost.containsAll(result) && result.containsAll(humanSetHumanNameContractsPost));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());						
		}		
	}
	
	
	@Test
	public void testGetContractHuman4() {
		try {
			String signature = "public void com.didana.codecontract.tests.Human.setHumanName(java.lang.String)";
			ArrayList<String> result = ContractHandler.getContracts(humanClass, preClass, signature, true);
			assertTrue(humanSetHumanNameContractsPre.containsAll(result) && result.containsAll(humanSetHumanNameContractsPre));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());						
		}		
	}
	
	@Test
	public void testGetContractHuman5() {
		try {
			String signature = "public java.lang.String com.didana.codecontract.tests.Human.getFullName()";
			ArrayList<String> result = ContractHandler.getContracts(humanClass, postClass, signature, true);
			assertTrue(humanGetFullNameContractsPost.containsAll(result) && result.containsAll(humanGetFullNameContractsPost));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());						
		}		
	}
	
	@Test
	public void testGetContractHuman6() {
		try {
			String signature = "private void com.didana.codecontract.tests.Human.privateSet()";
			ArrayList<String> result = ContractHandler.getContracts(humanClass, preClass, signature, false);
			assertTrue(humanPrivateSetPre.containsAll(result) && result.containsAll(humanPrivateSetPre));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());						
		}		
	}
	
	@Test
	public void testGetContractHuman7() {
		try {
			String signature = "protected void com.didana.codecontract.tests.Human.protectedSet()";
			ArrayList<String> result = ContractHandler.getContracts(humanClass, postClass, signature, false);
			assertTrue(humanProtectedSetPost.containsAll(result) && result.containsAll(humanProtectedSetPost));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());						
		}		
	}
	
	@Test
	public void testGetContractHuman8() {
		try {
			String signature = "void com.didana.codecontract.tests.Human.packageSet()";
			ArrayList<String> result = ContractHandler.getContracts(humanClass, postClass, signature, false);
			assertTrue(humanPackageSetPost.containsAll(result) && result.containsAll(humanPackageSetPost));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());						
		}		
	}
	
	@Test
	public void testGetContractHuman9() {
		try {
			String signature = "public void com.didana.codecontract.tests.Human.staticSet()";
			String result = ContractHandler.getStaticContracts(humanClass, postClass, signature);
			assertTrue(result.equals("isStaticPost == true"));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());						
		}		
	}
	
	@Test
	public void testGetContractHuman10() {
		try {
			String signature = "public void com.didana.codecontract.tests.Human.staticSet()";
			String result = ContractHandler.getStaticContracts(humanClass, preClass, signature);
			assertTrue(result.trim().equals("isStaticPre == true"));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());						
		}		
	}
	
	
	@Test
	public void testGetContractHuman11() {
		try {
			String signature = "public com.didana.codecontract.tests.Human()";
			ArrayList<String> result = ContractHandler.getConstructorContracts(humanClass, preClass, signature);
			assertTrue(humanConstructorPre.containsAll(result) && result.containsAll(humanConstructorPre));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());						
		}		
	}
	
	@Test
	public void testGetContractHuman12() {
		try {
			String signature = "public com.didana.codecontract.tests.Human(java.lang.String)";
			ArrayList<String> result = ContractHandler.getConstructorContracts(humanClass, postClass, signature);
			assertTrue(humanConstructorPost.containsAll(result) && result.containsAll(humanConstructorPost));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());						
		}		
	}	
	
	@Test
	public void testGetContractHuman13() {
		try {
			String signature = "public void com.didana.codecontract.tests.Human.setDescription(java.lang.String)";
			ArrayList<String> result = ContractHandler.getContracts(humanClass, postClass, signature, true);
			assertTrue(humanInvariantContracts.containsAll(result) && result.containsAll(humanInvariantContracts));
		} catch (ContractException ce) {
			fail("ContractException thrown." + ce.getMessage());						
		}		
	}
	
}
