package com.didana.codecontract.util;

import java.lang.annotation.Annotation;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

/**
 * Utility class that wraps the functionality of log4j package
 * and define methods for printing information about a class.
 * 
 * @author Diana Berberova
 *
 */
public class LogUtillity
{
	private Logger logger;
    private Class classInstance;
    private static boolean configured;

    static 
    {
        configured = false;
        if(!configured)
            BasicConfigurator.configure();
    }

    public LogUtillity(Class classInstance) {
        this.classInstance = classInstance;
        logger = Logger.getLogger(classInstance);
    }

    public void printFields() {
        try
        {
            Field declaredFields[] = classInstance.getDeclaredFields();
            Field fields[] = classInstance.getFields();
            logger.debug((new StringBuilder("CLASS: ")).append(classInstance.getName()).toString());
            for(int i = 0; i < declaredFields.length; i++) {
                logger.debug((new StringBuilder("DECLARED FIELDS: ")).append(declaredFields[i].getName()).toString());
            }
            for(int j = 0; j < fields.length; j++) {
                logger.debug((new StringBuilder("PUBLIC FIELDS: ")).append(fields[j].getName()).toString());
            }
            
        } catch(Exception e) {
    		logger.error("Unexpected exception during printing of object fields: ", e);
            e.printStackTrace();
        }
    }

    public void printMethods(Class classInstance)
    {
        try
        {
            Method methods[] = classInstance.getDeclaredMethods();
            logger.debug((new StringBuilder("CLASS: ")).append(classInstance.getName()).toString());
            for(int i = 0; i < methods.length; i++) {
                logger.debug((new StringBuilder("METHOD: ")).append(methods[i].toString()).toString());
            }
            
            Constructor constructor[] = classInstance.getDeclaredConstructors();
            logger.debug((new StringBuilder("CLASS: ")).append(classInstance.getName()).toString());
            for(int i = 0; i < constructor.length; i++) {
                logger.debug((new StringBuilder("CONSTRUCTOR: ")).append(constructor[i].toString()).toString());
            }
        }
        catch(Exception e) {
          logger.error("Unexpected exception during printing of object methods: ", e);
          e.printStackTrace();
        }
    }

    public void printAnnotation(Annotation annotation) {
        Method annotationMethods[] = annotation.annotationType().getMethods();
        logger.debug((new StringBuilder("Annotation methods count: ")).append(annotationMethods.length).toString());
        for(int j = 0; j < annotationMethods.length; j++) {
            logger.debug((new StringBuilder("Annotation method name: ")).append(annotationMethods[j].getName()).toString());
            if(annotationMethods[j].getName().equals("value")) {
                Method valueMethod = annotationMethods[j];
                
                try {
                    Object annotationValue = valueMethod.invoke(annotation, (Object[]) null);
                    (new Exception()).printStackTrace();
                    logger.debug((new StringBuilder("Annotation value: ")).append(annotationValue.toString()).toString());
                } catch(Exception e) {
            		logger.error("Unexpected exception during printing of annotations: ", e);
                    e.printStackTrace();
                }
            }
        }
    }

    public void debug(String message)
    {
        logger.debug(message);
    }

    public void debug(String message, Throwable t)
    {
        logger.debug(message, t);
    }

    public void error(String message)
    {
        logger.error(message);
    }

    public void error(String message, Throwable t)
    {
        logger.error(message, t);
    }
   
}
